@@ -1139,7 +1139,7 @@ def test_unorderable_types(testdir):
     """
     )
     result = testdir.runpytest()
-    assert "TypeError" not in result.stdout.str()
+    result.stdout.no_fnmatch_line("*TypeError*")
     assert result.ret == ExitCode.NO_TESTS_COLLECTED
 
 
@@ -1148,7 +1148,7 @@ def test_dont_collect_non_function_callable(testdir):
     """Test for issue https://github.com/pytest-dev/pytest/issues/331
 
     In this case an INTERNALERROR occurred trying to report the failure of
-    a test like this one because py test failed to get the source lines.
+    a test like this one because pytest failed to get the source lines.
     """
     testdir.makepyfile(
         """
@@ -1167,7 +1167,7 @@ def test_dont_collect_non_function_callable(testdir):
         [
             "*collected 1 item*",
             "*test_dont_collect_non_function_callable.py:2: *cannot collect 'test_a' because it is not a function*",
-            "*1 passed, 1 warnings in *",
+            "*1 passed, 1 warning in *",
         ]
     )
 
